import { describe, it, expect, afterEach } from '@jest/globals'
import path from 'path'
import vitePluginProxy from '../../src/index'
import logger from '../../src/logger'
import { closeAllServers, createTestServer, createViteServer, generateSelfsigned, getPort, http2Request, updateRequestManager } from '../util'

const PORT = getPort()
const SERVER_PORT = getPort()
const __dirname = path.dirname(new URL(import.meta.url).pathname)

describe('changeOrigin', function () {
  afterEach(async function () {
    await closeAllServers()
    updateRequestManager()
  })

  describe('http1 to http1', function () {
    it('is enabled', async function () {
      await createViteServer({
        root: __dirname,
        logLevel: 'silent',
        plugins: [vitePluginProxy({
          proxy: {
            '/api': {
              target: `http://localhost:${SERVER_PORT}`,
              changeOrigin: true
            }
          }
        })]
      }, PORT)

      const server = createTestServer()
      const promise = new Promise((resolve) => {
        server.get('/api/test', (req, res) => {
          res.send()
          resolve(req.headers)
        })
      })
      await server.listen({ port: SERVER_PORT })

      await http2Request(`http://localhost:${PORT}/api/test`)
      const headers = await promise
      expect(headers.host).toEqual(`localhost:${SERVER_PORT}`)
    })

    it('is disabled (default)', async function () {
      await createViteServer({
        root: __dirname,
        logLevel: 'silent',
        plugins: [vitePluginProxy({
          proxy: {
            '/api': {
              target: `http://localhost:${SERVER_PORT}`
            }
          }
        })]
      }, PORT)

      const server = createTestServer()
      const promise = new Promise((resolve) => {
        server.get('/api/test', (req, res) => {
          res.send()
          resolve(req.headers)
        })
      })
      await server.listen({ port: SERVER_PORT })

      await http2Request(`http://localhost:${PORT}/api/test`)
      const headers = await promise
      expect(headers.host).toEqual(`localhost:${PORT}`)
    })
  })

  describe('http2 to http1', function () {
    it('is enabled', async function () {
      await createViteServer({
        root: __dirname,
        server: { https: generateSelfsigned() },
        logLevel: 'silent',
        plugins: [vitePluginProxy({
          proxy: {
            '/api': {
              target: `https://localhost:${SERVER_PORT}`,
              changeOrigin: true,
              secure: false
            }
          }
        })]
      }, PORT)

      const server = createTestServer({
        http2: true,
        https: generateSelfsigned()
      })
      const promise = new Promise((resolve) => {
        server.get('/api/test', (req, res) => {
          res.send()
          resolve(req.headers)
        })
      })
      await server.listen({ port: SERVER_PORT })

      await http2Request(`https://localhost:${PORT}/api/test`, {
        rejectUnauthorized: false
      })
      const headers = await promise
      expect(headers.host).toEqual(`localhost:${SERVER_PORT}`)
    })

    it('is disabled (default)', async function () {
      // seems like https servers need more time to close ...
      const SERVER_PORT = getPort()
      const PORT = getPort()
      await createViteServer({
        root: __dirname,
        server: { https: generateSelfsigned() },
        logLevel: 'silent',
        plugins: [vitePluginProxy({
          proxy: {
            '/api': {
              target: `https://localhost:${SERVER_PORT}`,
              secure: false
            }
          }
        })]
      }, PORT)

      const server = createTestServer({
        http2: true,
        https: generateSelfsigned()
      })
      const promise = new Promise((resolve) => {
        server.get('/api/test', (req, res) => {
          res.send()
          resolve(req.headers)
        })
      })
      await server.listen({ port: SERVER_PORT })

      await http2Request(`https://localhost:${PORT}/api/test`, {
        rejectUnauthorized: false
      })
      const headers = await promise
      expect(headers.host).toEqual(`localhost:${PORT}`)
    })
  })
})
