import { describe, it, expect, afterEach } from '@jest/globals'
import path from 'path'
import vitePluginProxy from '../../src/index'
import { closeAllServers, createTestServer, createViteServer, generateSelfsigned, getPort, http2Request, updateRequestManager } from '../util'

const PORT = getPort()
const SERVER_PORT = getPort()
const __dirname = path.dirname(new URL(import.meta.url).pathname)

describe('simple proxy', function () {
  afterEach(async function () {
    await closeAllServers()
    updateRequestManager()
  })

  it('works', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        proxy: {
          '/api': {
            target: `http://localhost:${SERVER_PORT}`
          }
        }
      })]
    }, PORT)

    const server = createTestServer()
    server.get('/api/test', (req, res) => {
      res.send({ hello: 'world' })
    })
    await server.listen({ port: SERVER_PORT })

    const { data, httpVersion } = await http2Request(`http://localhost:${PORT}/api/test`)
    expect(data).toEqual({ hello: 'world' })
    expect(httpVersion).toEqual('1.1')
  })

  it('works with http2', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      server: {
        https: generateSelfsigned()
      },
      plugins: [vitePluginProxy({
        proxy: {
          '/api': {
            target: `https://localhost:${SERVER_PORT}`,
            secure: false
          }
        }
      })]
    }, PORT)

    const server = createTestServer({
      http2: true,
      https: generateSelfsigned()
    })
    server.get('/api/test', (req, res) => {
      res.send({ hello: 'world' })
    })
    await server.listen({ port: SERVER_PORT })

    const { data, httpVersion } = await http2Request(`https://localhost:${PORT}/api/test`, {
      rejectUnauthorized: false
    })
    expect(data).toEqual({ hello: 'world' })
    expect(httpVersion).toEqual('2.0')
  })

  it('regular expressions', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        proxy: {
          '^/(foo|bar)': {
            target: `http://localhost:${SERVER_PORT}`
          }
        }
      })]
    }, PORT)

    const server = createTestServer()
    server.get('/api/test', (req, res) => {
      res.send({ hello: 'world' })
    })
    server.get('/foo/test', (req, res) => {
      res.send({ hello: 'foo' })
    })
    server.get('/bar/test', (req, res) => {
      res.send({ hello: 'bar' })
    })
    await server.listen({ port: SERVER_PORT })

    const { data } = await http2Request(`http://localhost:${PORT}/foo/test`)
    expect(data).toEqual({ hello: 'foo' })
    const { data: data2 } = await http2Request(`http://localhost:${PORT}/bar/test`)
    expect(data2).toEqual({ hello: 'bar' })
  })

  it('rewrite rule', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        proxy: {
          '/api': {
            target: `http://localhost:${SERVER_PORT}`,
            rewrite: () => '/appsuite/api/test'
          }
        }
      })]
    }, PORT)

    const server = createTestServer()
    server.get('/appsuite/api/test', (req, res) => {
      res.send({ hello: 'world' })
    })
    await server.listen({ port: SERVER_PORT })

    const { data } = await http2Request(`http://localhost:${PORT}/api/test`)
    expect(data).toEqual({ hello: 'world' })
  })

  it('prepends a path', async function () {
    await createViteServer({
      root: __dirname,
      logLevel: 'silent',
      plugins: [vitePluginProxy({
        proxy: {
          '/api': {
            target: `http://localhost:${SERVER_PORT}`,
            prependPath: '/appsuite'
          }
        }
      })]
    }, PORT)

    const server = createTestServer()
    server.get('/appsuite/api/test', (req, res) => {
      res.send({ hello: 'world' })
    })
    await server.listen({ port: SERVER_PORT })

    const { data } = await http2Request(`http://localhost:${PORT}/api/test`)
    expect(data).toEqual({ hello: 'world' })
  })
})
